@@ -1,4 +1,4 @@
-# Copyright 1999-2014 Gentoo Foundation
+# Copyright 1999-2018 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 import logging
@@ -26,9 +26,11 @@ def create_depgraph_params(myopts, myaction):
 	# ignore_soname_deps: ignore the soname dependencies of built
 	#   packages, so that they do not trigger dependency resolution
 	#   failures, or cause packages to be rebuilt or replaced.
+	# ignore_world: ignore the @world package set and its dependencies
 	# with_test_deps: pull in test deps for packages matched by arguments
 	# changed_deps: rebuild installed packages with outdated deps
 	# changed_deps_report: report installed packages with outdated deps
+	# changed_slot: rebuild installed packages with outdated SLOT metadata
 	# binpkg_changed_deps: reject binary packages with outdated deps
 	myparams = {"recurse" : True}
 
@@ -56,16 +58,24 @@ def create_depgraph_params(myopts, myaction):
 		myparams["selective"] = True
 		return myparams
 
+	if myopts.get('--ignore-world') is True:
+		myparams['ignore_world'] = True
+
 	rebuild_if_new_slot = myopts.get('--rebuild-if-new-slot')
 	if rebuild_if_new_slot is not None:
 		myparams['rebuild_if_new_slot'] = rebuild_if_new_slot
 
+	changed_slot = myopts.get('--changed-slot') is True
+	if changed_slot:
+		myparams["changed_slot"] = True
+
 	if "--update" in myopts or \
 		"--newrepo" in myopts or \
 		"--newuse" in myopts or \
 		"--reinstall" in myopts or \
 		"--noreplace" in myopts or \
 		myopts.get("--changed-deps", "n") != "n" or \
+		changed_slot or \
 		myopts.get("--selective", "n") != "n":
 		myparams["selective"] = True
 
